%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
%-----Introduction to Heavy Good Electric Vehicle (HGEV) Depot Charging----
%-----------------------Demand Modeling Software---------------------------
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
% Introducing the "Heavy Good Electric Vehicle (HGEV) Charging Modelling": 
% This software package helps you model the charging demand profile of a fleet 
% of Heavy Good Electric Vehicles (HGEVs) operating in a depot style environment.
%--------------------------------------------------------------------------
% This is the main controlling part of depot load modeling, it accepts depot 
% and vehicle specifications, and calls the relevant functions to complete 
% the modeling of the load profile
%**************************************************************************
clc; clear;close all;
%%*************************************************************************
%--------------------------------------------------------------------------
%--------------------------Definition of Input Data------------------------
%--------------------------------------------------------------------------
%%*************************Fleet Specifications****************************
FleetSize=50;                                    % number of the vehicles (change for different fleet size) 
BatteryCapacity = 400;                           % battery capacity [kWh]
SpecificConsumption   = 1.8;                     % specific energy consumption[kWh/Mile]
%%*************************Depot Specifications****************************
ChargingPower = 50;                               % power rating of a charger[kW]
DepotFactor=0.7;                                  % depot limiting factor (is used to set a cap for the total amount of power(depot limit) available in the depot, the maximum is "1")
                                                  % Note: for FleetSize = 1; DepotFactor must be equal to 1
%%***************************Charging Sart Time****************************
StartHour=19;                                     % input hours (24 hour format)
StartMinute=0;                                    % input minutes
StartSecond=0;                                    % input seconds
%%*************************************************************************
%--------------------------------------------------------------------------
%---------------The definition of Input Data has been finalised.-----------
%-----------------It is time to proceed with running the model.------------
%--------------------------------------------------------------------------
%**************************************************************************
DepotLimit= floor((FleetSize*DepotFactor))*ChargingPower;     % Power Limit of depot 
%**************************************************************************
FileName = sprintf('FleetLoadProfile_%s.xlsx', datestr(now,'yyyy-mm-dd_HH-MM')); % creates an Excel file where the results are saved
%%*********Gets the traveled distanaces for the fleet***
FleetTravel= FuncTraveledDistance(FleetSize);        % "FuncTraveledDistance" will return the fleet daily traveled distance
%%*********Gets the SoCs of  fleet (at the end of daily operation) and stores in the file***
FleetStateOfCharge = FuncFleetStateOfChargeSorted(BatteryCapacity,SpecificConsumption,FleetTravel); % "FuncFleetStateOfChargeSorted" will return the SOC
writematrix('State of Charge',FileName,'Sheet',3,'Range','A1')  ;                                    % Writes the header for the data on the relevent column
writematrix((FleetStateOfCharge'),FileName,'Sheet',3,'Range','A2') ;                                 % Writes the header for the data on the relevent column
%%*********Call the Vehcile Charging Function in two parts (based on the depot limit)*********************
StartTimeSetting=(StartHour+StartMinute/60+StartSecond/3600)*3600; %{two consecutive days (48 hours) time is considered for charging analysis, but for the purpose of illustration it is mapped to 24-hour format}
TotalPower=0;
%%*********Part 1: Alocating the first set of vehicles to be charged**************
FirstFleetSet=floor(DepotLimit/ChargingPower);           % calculates the first set of vehicles to be charged to be charged by the available power at the depot
RemainingFleet=FleetSize-FirstFleetSet;                  % calculates the remaining number of vehicles that are going to be charges as soon as a charger becomes available
SoCmaxTimeRecorederFirstSet=[];                          % the matrix to store the time of maximum SOC of the vehicles 
for Vehicle=1:FirstFleetSet   
    VehicleName = strcat( 'Vehicle_',num2str(Vehicle) );
    Alphabet = reshape('A':'Z',[],1);
    Column = num2cell(Alphabet(1+rem(Vehicle-1,26)));
    v = fix((Vehicle-1)/26);
    while any(v), Column(v>0) = strcat(Alphabet(1+rem(v(v>0)-1,26)),Column(v>0)); v = fix((v-1)/26); end
    Column=Column{1}        
    writematrix(VehicleName,FileName,'Sheet',1,'Range',[Column,num2str(1)]) % Writes the header for the data on the relevent column in the file  
    SOC=FleetStateOfCharge(Vehicle);
    StartTime=StartTimeSetting; 
    EndTime=48*3600;                                                         
    [VehicleLoadXaxis, VehicleLoad, SoCmaxTimeRecoreder]=FuncVehicleCharging(FileName,Column, ChargingPower, SOC, BatteryCapacity, StartTime, EndTime);
    TotalPower=TotalPower+VehicleLoad;
    SoCmaxTimeRecorederFirstSet=[SoCmaxTimeRecorederFirstSet SoCmaxTimeRecoreder];
end
%%% Part 2:  ******************************************
%% RecordingTimes=SoCmaxTimeRecorederFirstSet;          
StartTimeRemaining= min(SoCmaxTimeRecorederFirstSet); % the time that the vehicles of the second set are to start charging
SoCmaxTimeRecorederAll=SoCmaxTimeRecorederFirstSet;
for Vehicle=FirstFleetSet+1:FleetSize    
    VehicleName = strcat( 'Vehicle_',num2str(Vehicle));
    Alphabet = reshape('A':'Z',[],1);
    Column = num2cell(Alphabet(1+rem(Vehicle-1,26)));
    v = fix((Vehicle-1)/26);
    while any(v), Column(v>0) = strcat(Alphabet(1+rem(v(v>0)-1,26)),Column(v>0)); v = fix((v-1)/26); end
    Column=Column{1}   
    writematrix(VehicleName,FileName,'Sheet',1,'Range',[Column,num2str(1)]) % Writes the header for the data on the relevent column
    SOC=FleetStateOfCharge(Vehicle);   
    if (StartTimeRemaining >= 3600*24)
          StartTimeRem=StartTimeRemaining-24*3600+1; 
    else
          StartTimeRem=StartTimeRemaining+1; 
    end   
    SoCmaxTimeRecorederAll(SoCmaxTimeRecorederAll==min(SoCmaxTimeRecorederAll))=10e10;   
    EndTimeRem=48*3600;
    [VehicleLoadXaxis, VehicleLoad, SoCmaxTimeRecoreder]=FuncVehicleCharging(FileName,Column, ChargingPower, SOC, BatteryCapacity, StartTimeRem, EndTimeRem);
    TotalPower=TotalPower+VehicleLoad;
    %********finding the minimum Starting time ********** 
    SoCmaxTimeRecorederAll=[SoCmaxTimeRecorederAll SoCmaxTimeRecoreder];
    StartTimeRemaining= min(SoCmaxTimeRecorederAll);
end
writematrix('Demand',FileName,'Sheet',2,'Range','A1')     % writes the header for the data in the relevent column
writematrix((TotalPower),FileName,'Sheet',2,'Range','A2') % stores the "Total Power" (demand) in the relevent column
%%*************************************************************************
%%************************Calls Plotting fucntion *************************
% this function plots the basic load profile of the depot charging , and
% optiinally avergage load profile (selectable options within the function 'FuncLoadProfilePlotting')
FuncLoadProfilePlotting(VehicleLoadXaxis, TotalPower, FileName) % calling the plotting function
%%*************************************************************************